<?php
/**
 * @file
 * Contains Validation form for OTP Verification Module.
 */

/**
 * Validate user function.
 */
function otp_otp_form($form, &$form_state) {
  $token = drupal_get_token();
  $uid = '';
  if (!user_access('administer site configuration')) {
    if (isset($_SESSION['otp_user_register_uid'])) {
      $uid = $_SESSION['otp_user_register_uid'];
    }
    else {
      $query_params = drupal_get_query_parameters();
      if (isset($query_params['u'])) {
        $user = db_select('users', 'u')
          ->fields('u', ['uid'])
          ->condition('uuid', $query_params['u'])
          ->execute()
          ->fetchCol();
        if (!empty($user)) {
          $uid = $user[0];
        }
      }
    }
  }
  $form['user_id'] = array(
    '#type' => 'textfield',
    '#title' => 'User id',
    '#default_value' => $uid,
    '#access' => user_access('administer site configuration')
  );

  $form['user_email'] = array(
    '#type' => 'textfield',
    '#title' => 'User Email',
    '#access' => $uid ? user_access('administer site configuration') : TRUE
  );
  $form['otp'] = array(
    '#name' => 'otp',
    '#type' => 'textfield',
    '#attributes' => array(
      'autofocus' => 'autofocus',
    ),
    '#title' => t("Please enter the 6-digit verification code we've just sent to your email address"),
    '#required' => TRUE,
    '#default_value' => '',
    '#size' => 60,
    '#description' => t("If you can't find the code in your main email folder, search your spam/junk/promotions folder",
      array(
        '%email' => (isset($_SESSION['otp_user_register_form_state']['values']['mail']) ? $_SESSION['otp_user_register_form_state']['values']['mail'] : 'email')
      )
    ),
    '#maxlength' => 15,
  );
  $form['actions'] = array('#type' => 'actions');
  $form['actions']['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Verify'),
    '#validate' => array('otp_otp_form_validate'),
    '#submit' => array('otp_otp_form_submit')
  );
  $form['actions']['resend'] = array(
    '#type' => 'submit',
    '#value' => t('Resend verification code'),
    '#limit_validation_errors' => array(),
    '#submit' => array('otp_resend_otp'),
  );
  $form['actions']['cancel'] = array(
    '#markup' => l(t('Cancel'), 'user/register'),
  );
  if (isset($_GET['destination'])) {
    $form['destination'] = array(
      '#type' => 'hidden',
      '#value' => $_GET['destination']
    );
  }
  $form['need_help'] = [
    '#markup' => "<div class='help-block'>Need help? Email <a href='mailto:info@gokhalemethod.com'>info@gokhalemethod.com</a>, or call us on <a href='tel:1-844-777-0440'>1-844-777-0440</a> (8am–5pm daily, US Pacific Time)</div>",
    '#weight' => 1000
  ];

  return $form;
}

/**
 * Validates otp entered by the user.
 */
function otp_otp_form_validate($form, &$form_state) {
  if (user_access('administer site configuration')) {
    if (trim($form_state['values']['user_id']) == '' && trim($form_state['values']['user_email']) == '') {
      form_set_error('user_id','Please enter user id or user email', 'error');
    }
  }
  if (trim($form_state['values']['user_id']) == '') {
    $user = user_load_by_mail($form_state['values']['user_email']);
  }
  else
    $user = user_load($form_state['values']['user_id']);
  if (!flood_is_allowed('user_otp_submit', 15, 60*60*3)) { // 3 hours
    // We allow only 15 submits in 3 hours.
    form_set_error('otp', t("It looks like you have tried to validate otp many times, please wait for few hours and retry"));
    return FALSE;
  }
  flood_register_event('user_otp_submit', 60*60*3);

  if ($user->login) {
    form_set_error('otp', t("It looks like you've already completed this step. If you're trying to !login_url and/or reset your password.", ['!login_url' => l('login to your account click here', 'user/login')]));
  }
  if ($user->data['otp_user_register_random_otp'] != md5($form_state['values']['otp'])) {
    form_set_error('otp', t('You entered an incorrect activation code. Please enter the activation code that we sent to your email %email.', array('%email' => $user->mail)));
  } elseif (!isset($user->data['otp_user_register_random_otp_time']) || $user->data['otp_user_register_random_otp_time'] < (time() - 24 * 60 * 60)) { //24*60*60
    form_set_error('otp', t("The activation code you entered has expired. !resend_verification",  ['!resend_verification' => l('Click here to send a new code to your email address.', 'user/register/otp/resend/' . $user->uuid)]));
  }
}

/**
 * OTP matched now register user.
 */
function otp_otp_form_submit($form, $form_state) {
  $destination = isset($form_state['values']['destination']) ? $form_state['values']['destination'] : '';
  if (trim($form_state['values']['user_id']) == '') {
    $user = user_load_by_mail($form_state['values']['user_email']);
  }
  else
    $user = user_load($form_state['values']['user_id']);
  $user->status = TRUE;
  user_save($user);
  if (user_access('administer site configuration')) {
    drupal_set_message(t('User account activated'));
    return;
  }
  $login_form_state['uid'] = $form_state['values']['user_id'];
  user_login_submit(array(), $login_form_state);
  unset($_SESSION['otp_user_register_uid']);
  drupal_set_message(t('Welcome, You can now register for events and view exclusive content.'));
  if ($destination) {
    drupal_goto($destination);
  }
  else {
    drupal_goto();
  }
}

/**
 * Resend otp.
 */
function otp_resend_otp(&$form, $form_state) {
  $user = user_load($form['user_id']['#default_value']);
  $otp_send = _otp_generate_otp($user, TRUE);
  if (is_array($otp_send) && $otp_send['email_send']) {
    drupal_set_message(t('A new verification code has been sent to %email', array('%email' => $user->mail)));
  }
  elseif (!$otp_send) {
    drupal_set_message(t("You've reached the maximum number of attempts. Please contact our Customer Support Team at 888-557-6788 for assistance with creating your account."), "error");
  }
  else {
    drupal_set_message(t('Some error occurred in sending OTP'), 'error');
  }
}

function otp_resend_otp_link($uuid) {
  $user = db_select('users', 'u')
    ->fields('u', ['uid'])
    ->condition('uuid', $uuid)
    ->execute()
    ->fetchCol();
  if (!empty($user)) {
    $uid = $user[0];
    $form['user_id']['#default_value'] = $uid;
    $form_state = [];
    otp_resend_otp($form, $form_state);
  }
  drupal_goto('user/register/otp', ['query' => ['u' => $uuid]]);
}
